require "influxdb/rails/middleware/subscriber"

module InfluxDB
  module Rails
    module Middleware
      class RequestSubscriber < Subscriber # :nodoc:
        def call(_name, start, finish, _id, payload) # rubocop:disable Metrics/MethodLength
          return unless enabled?

          #ts = InfluxDB.convert_timestamp(finish.utc, configuration.time_precision)
          ts = InfluxDB.convert_timestamp(start.utc, configuration.time_precision)
          tags = tags(payload)
          x_request_id = "AL_NONE"
          al_request_id = "AL_NONE"
          al_request_guid = "AL_NONE"
          al_source = "AL_NONE"
          al_request_client_id = "AL_NONE"

          x_request_id = payload[:headers][:HTTP_X_REQUEST_ID] if(payload[:headers] && payload[:headers][:HTTP_X_REQUEST_ID])
          al_request_id = payload[:headers][:HTTP_AL_REQUEST_ID] if(payload[:headers] && payload[:headers][:HTTP_AL_REQUEST_ID])
          #al_request_id = Thread.current["al_request_id"] if(Thread.current["al_request_id"])
          al_request_guid = payload[:headers][:HTTP_AL_REQUEST_GUID] if(payload[:headers] && payload[:headers][:HTTP_AL_REQUEST_GUID])
          #al_request_guid = Thread.current["al_request_guid"] if(Thread.current["al_request_guid"])
          al_source = payload[:headers][:HTTP_AL_SOURCE] if(payload[:headers] && payload[:headers][:HTTP_AL_SOURCE])
          al_request_client_id = payload[:headers][:HTTP_AL_REQUEST_CLIENT_ID] if(payload[:headers] && payload[:headers][:HTTP_AL_REQUEST_CLIENT_ID])

          al_source = Thread.current["al_source"]
          al_request_client_id = Thread.current["al_request_client_id"]

          begin
            series(payload, start, finish).each do |series_name, value|
              InfluxDB::Rails.client.write_point \
                series_name,
                values:    { value: value, x_request_id: x_request_id, al_request_id: al_request_id, al_request_guid: al_request_guid, al_request_source_id: al_source, al_request_client_id: al_request_client_id },
                tags:      tags,
                timestamp: ts
            end
          rescue StandardError => e
            log :error, "[InfluxDB::Rails] Unable to write points: #{e.message}"
          ensure
            InfluxDB::Rails.current.reset
          end
        end

        private

        def series(payload, start, finish)
          {
            configuration.series_name_for_controller_runtimes => ((finish - start) * 1000).ceil,
            configuration.series_name_for_view_runtimes       => (payload[:view_runtime] || 0).ceil,
            configuration.series_name_for_db_runtimes         => (payload[:db_runtime] || 0).ceil,
          }
        end

        def tags(payload)
          tags = {
            method:      "#{payload[:controller]}##{payload[:action]}",
            status:      payload[:status],
            format:      payload[:format],
            http_method: payload[:method],
            host:      Socket.gethostname,
            app_name:    configuration.application_name,
          }
          super(tags)
        end
      end
    end
  end
end
